@@ -63,12 +63,17 @@
 
 ;;; Change log:
 
+;;; 2002/5/22 $BL5JQ49<-=q!"J8@a@Z$j<-=q$NDI2C=hM}$N=$@5(B
+;;; 2002/5/22 $B@\F,8l%Q%i%a!<%?(B prefix-flag $B$N@_DjCM$N=$@5(B
+;;; 2002/5/16 XEmacs $B$G%f!<%6<-=q$N3JG<>l=jJQ99$,@5$7$/9T$J$($J$$LdBj$NBP1~(B
+;;; 2002/5/16 $B%f!<%6<-=q$N%/%i%$%"%s%HE>AwF0:n$K$*$1$kIT6q9g=$@5(B
+;;; 2002/5/16 $BM=B,%b!<%I;~$K5UJQ49F0:n$,@5$7$/9T$J$($J$$LdBj$KBP1~(B
 ;;; 2001/9/30 v1.01 (bug fix)
 ;;; 2001/5/30 First Release
 
 ;;; Code:
 
-;;; Last modified date: 2001/9/30
+;;; Last modified date: 2002/5/22
 
 (require 'wnn7egg-edep)
 (if (featurep 'xemacs)
@@ -325,7 +330,8 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
 	((eq c 'WNN_FI_SYSTEM_DICT) 6)
 	((eq c 'WNN_FI_USER_DICT) 7)
 	((eq c 'WNN_GROUP_DICT)   9)
-	((eq c 'WNN_MERGE_DICT)  10)))
+	((eq c 'WNN_MERGE_DICT)  10)
+	((eq c 'WNNDS_FILE_READ_ERROR) 90)))
 ;;	((eq c 'DIC_NO_TEMPS)   ?\x3f))) ; cannot eval on xemacs
 
 ;; Retern value of system-name may differ from hostname.
@@ -709,7 +715,9 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
 	      ((error quit))))
 	  (when proc
 	    (process-kill-without-query proc)
-	    (set-process-coding-system proc 'no-conversion 'no-conversion)
+	    (if (featurep 'xemacs)
+		(set-process-coding-system proc 'binary 'binary)
+	      (set-process-coding-system proc 'no-conversion 'no-conversion))
 	    (set-process-sentinel proc 'wnn7-comm-sentinel)
 	    (set-marker-insertion-type (process-mark proc) t))
 	  proc)
@@ -938,9 +946,9 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
 	(wnn7-dicinfo-mode (car info) 0)
       (WNN-const DIC_RDONLY))))
 
-(defun wnn7-server-notrans-dict-add (dname prior drw &optional dpass)
+(defun wnn7-server-bmodify-dict-add (dname prior drw &optional dpass)
   (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
-	 (type (WNN-const NOTRANS_LEARN))
+	 (type (WNN-const BMODIFY_LEARN))
 	 (did (wnn7rpc-get-autolearning-dic env type))
 	 (param-list (wnn7-make-henkan-parameter))
 	 (name 'bunsetsugiri)
@@ -952,12 +960,12 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
 				((or (eq mode 1) (eq mode t))   1)
 				(t (egg:error "%s: Wrong type argument" name))))
       (wnn7rpc-set-conversion-env-param env vmask param-list))
-    (wnn7env-set-notrans env (wnn7-get-autolearning-dic-mode
-			     env (WNN-const NOTRANS_LEARN)))))
+    (wnn7env-set-bmodify env (wnn7-get-autolearning-dic-mode
+			     env (WNN-const BMODIFY_LEARN)))))
 
-(defun wnn7-server-bmodify-dict-add (dname prior drw &optional dpass)
+(defun wnn7-server-notrans-dict-add (dname prior drw &optional dpass)
   (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
-	 (type (WNN-const BMODIFY_LEARN))
+	 (type (WNN-const NOTRANS_LEARN))
 	 (did (wnn7rpc-get-autolearning-dic env type))
 	 (param-list (wnn7-make-henkan-parameter))
 	 (name 'muhenkan)
@@ -969,8 +977,8 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
 				((or (eq mode 1) (eq mode t))   1)
 				(t (egg:error "%s: Wrong type argument" name))))
       (wnn7rpc-set-conversion-env-param env vmask param-list))
-    (wnn7env-set-bmodify env (wnn7-get-autolearning-dic-mode
-			     env (WNN-const BMODIFY_LEARN)))))
+    (wnn7env-set-notrans env (wnn7-get-autolearning-dic-mode
+			     env (WNN-const NOTRANS_LEARN)))))
 
 (defun wnn7-temporary-dic-add (env rev)
   (let ((result (wnn7rpc-temporary-dic-loaded env)))
@@ -997,7 +1005,7 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
      ((numberp (setq dpass (wnn7rpc-read-passwd-file dpass)))
       (message "%s" (wnn7rpc-get-error-message (- dpass)))
       nil)
-     ((setq did (wnn7-open-dictionary env nil dname t "" dpass "" t))
+     ((setq did (wnn7-open-dictionary env nil dname t "" dpass "" nil))
       (if (and (>= (setq did (wnn7rpc-set-dictionary env did -1 prior drw drw
 						    dpass "" rev))
 		   0)
@@ -1109,10 +1117,8 @@ wnn-uniq-kanji: Use only one among same kanji candidates.")
 	(name 'prefix)
 	vmask)
     (setq vmask (lsh 1 5))
-    (aset param-list 5 (cond ((or (eq mode -1) (eq mode 'regulation)) -1)
-			     ((or (eq mode  0) (eq mode 'no))          0)
-			     ((or (eq mode  1) (eq mode 'yes))         1)
-			     ((or (eq mode 1) (eq mode t))   1)
+    (aset param-list 5 (cond ((or (eq mode  0) (eq mode 'hiragana)) 0)
+			     ((or (eq mode  1) (eq mode 'kanji))    1)
 			     (t (egg:error "%s: Wrong type argument" name))))
     (wnn7rpc-set-conversion-env-param env vmask param-list)))
 
@@ -1341,6 +1347,8 @@ On failure, return negative error code."
 	      (y-or-n-p (format (wnn7-msg-get 'no-dir)
 				(file-name-directory path))))
 	  (make-directory local-name t)
+	  ;; server $BB&$N%G%#%l%/%H%j$b:n@.$7$F$*$/(B
+	  (wnn7-create-directory env (concat wnn7-usr-dic-dir "/foo") t)
 	  (if (file-directory-p local-name)
 	      (progn
 		(message (wnn7-msg-get 'create-dir) path)
@@ -1377,7 +1385,8 @@ On failure, return negative error code."
       (message "Wnn: cannot omit dictionary name")
       nil)
      ((>= dic-id 0) dic-id)
-     ((or (null rw) (/= dic-id (- (WNN-const NO_EXIST))))
+     ((or (null rw) (and (/= dic-id (- (WNN-const NO_EXIST)))
+                        (/= dic-id (- (WNN-const WNNDS_FILE_READ_ERROR)))))
       (message (wnn7-msg-get 'no-dict1)
 	       name (wnn7rpc-get-error-message (- dic-id)))
       nil)
@@ -1411,7 +1420,8 @@ On failure, return negative error code."
     (cond
      ((null freq-id) -1)
      ((>= freq-id 0) freq-id)
-     ((or (null rw) (/= freq-id (- (WNN-const NO_EXIST))))
+     ((or (null rw) (and (/= freq-id (- (WNN-const NO_EXIST)))
+			 (/= freq-id (- (WNN-const WNNDS_FILE_READ_ERROR)))))
       (message (wnn7-msg-get 'no-freq1)
 	       name (wnn7rpc-get-error-message (- freq-id)))
       nil)
@@ -1423,7 +1433,7 @@ On failure, return negative error code."
       (setq freq-id (wnn7-open-file env name))
       (if (>= freq-id 0)
 	  freq-id
-	(message "%s" (wnn7rpc-get-error-message (- dic-id)))
+	(message "%s" (wnn7rpc-get-error-message (- freq-id)))
 	nil)))))
 
 (defun wnn7-create-frequency (env fi dic-id name comment passwd)
@@ -1970,7 +1980,7 @@ On failure, return negative error code."
 (defvar egg-predict-status nil)
 
 (defun wnn7-server-predict-init ()
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-init env)))
     (if (= result 0)
 	(progn
@@ -1983,7 +1993,7 @@ On failure, return negative error code."
       nil)))
 
 (defun wnn7-server-predict-free ()
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-free env)))
     (setq egg-predict-status nil)
     (if (= result 0)
@@ -1992,7 +2002,7 @@ On failure, return negative error code."
       nil)))
 
 (defun wnn7-server-predict-start (str)
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-yosoku env str)))
     (if (numberp result)
 	(if (= result -3011)
@@ -2002,59 +2012,60 @@ On failure, return negative error code."
       result)))
 
 (defun wnn7-server-predict-toroku (&optional kakutei-str)
-  (let* ((bunlist wnn7-bun-list)
-	 (head (car bunlist))
-	 env ykouho yklist result)
-    (if kakutei-str
-	;; before convert
-	(setq env (if env-normal wnn7-env-norm wnn7-env-rev)
-	      ykouho (vector kakutei-str
-			     (length (encode-coding-string  
-				      (comm-format-truncate-after-null 
-				       kakutei-str) egg-mb-euc))
-			     kakutei-str
-			     (length (encode-coding-string  
-				      (comm-format-truncate-after-null 
-				       kakutei-str) egg-mb-euc))
-			     0)
-	      yklist (cons ykouho yklist))
-      ;; after convert
-      (setq env (wnn7-bunsetsu-get-env head))
-      (let (bun bunsetsu)
-	(while bunlist
-	  (setq bun (car bunlist)
-		bunsetsu nil)
-	  (while bun
-	    (setq bunsetsu (cons bun bunsetsu)
-		  bunlist (cdr bunlist)
-		  bun (and (wnn7-bunsetsu-get-dai-continue bun)
-			   bunlist
-			   (car bunlist))))
-	  (setq bunsetsu (nreverse bunsetsu))
-	  (when bunsetsu
-	    (setq ykouho (vector (wnn7-get-major-bunsetsu-source bunsetsu)
-				 (length (encode-coding-string  
-					  (comm-format-truncate-after-null 
-					   (mapconcat 'wnn7-bunsetsu-get-yomi 
-						      bunsetsu ""))
-					  egg-mb-euc))
-				 (wnn7-get-major-bunsetsu-converted bunsetsu)
-				 (length (encode-coding-string  
-					  (comm-format-truncate-after-null
-					   (mapconcat 'wnn7-bunsetsu-get-converted 
-						      bunsetsu ""))
-					  egg-mb-euc))
-				 (wnn7-bunsetsu-get-hinshi (car (nreverse bunsetsu))))
-		  yklist (cons ykouho yklist)))))
-      (setq yklist (nreverse yklist)))
-    (if (= (setq result (wnn7rpc-yosoku-toroku env (length yklist) yklist)) 0)
-	t
-      (if (not (eq result -3011))
-	  (egg:error "%s" (wnn7rpc-get-error-message (- result))))
-      nil)))
+  (when env-normal
+    (let* ((bunlist wnn7-bun-list)
+	   (head (car bunlist))
+	   (env wnn7-env-norm)
+	   ykouho yklist result)
+      (if kakutei-str
+	  ;; before convert
+	  (setq ykouho (vector kakutei-str
+			       (length (encode-coding-string  
+					(comm-format-truncate-after-null 
+					 kakutei-str) egg-mb-euc))
+			       kakutei-str
+			       (length (encode-coding-string  
+					(comm-format-truncate-after-null 
+					 kakutei-str) egg-mb-euc))
+			       0)
+		yklist (cons ykouho yklist))
+	;; after convert
+	;;(setq env (wnn7-bunsetsu-get-env head))
+	(let (bun bunsetsu)
+	  (while bunlist
+	    (setq bun (car bunlist)
+		  bunsetsu nil)
+	    (while bun
+	      (setq bunsetsu (cons bun bunsetsu)
+		    bunlist (cdr bunlist)
+		    bun (and (wnn7-bunsetsu-get-dai-continue bun)
+			     bunlist
+			     (car bunlist))))
+	    (setq bunsetsu (nreverse bunsetsu))
+	    (when bunsetsu
+	      (setq ykouho (vector (wnn7-get-major-bunsetsu-source bunsetsu)
+				   (length (encode-coding-string  
+					    (comm-format-truncate-after-null 
+					     (mapconcat 'wnn7-bunsetsu-get-yomi 
+							bunsetsu ""))
+					    egg-mb-euc))
+				   (wnn7-get-major-bunsetsu-converted bunsetsu)
+				   (length (encode-coding-string  
+					    (comm-format-truncate-after-null
+					     (mapconcat 'wnn7-bunsetsu-get-converted 
+							bunsetsu ""))
+					    egg-mb-euc))
+				   (wnn7-bunsetsu-get-hinshi (car (nreverse bunsetsu))))
+		    yklist (cons ykouho yklist)))))
+	(setq yklist (nreverse yklist)))
+      (if (= (setq result (wnn7rpc-yosoku-toroku env (length yklist) yklist)) 0)
+	  t
+	(if (not (eq result -3011))
+	    (egg:error "%s" (wnn7rpc-get-error-message (- result))))
+	nil))))
       
 (defun wnn7-server-predict-selected-cand (pos)
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-selected-cand env pos)))
     (if (>= result 0)
 	result
@@ -2062,7 +2073,7 @@ On failure, return negative error code."
       nil)))
   
 (defun wnn7-server-predict-delete-cand (pos)
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-delete-cand env pos)))
     (if (= result 0)
 	t
@@ -2070,7 +2081,7 @@ On failure, return negative error code."
       nil)))
   
 (defun wnn7-server-predict-cancel-toroku ()
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-cancel-latest-toroku env)))
     (if (= result 0)
 	t
@@ -2078,7 +2089,7 @@ On failure, return negative error code."
       nil)))
   
 (defun wnn7-server-predict-reset-connective ()
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-reset-pre-yosoku env)))
     (if (= result 0)
 	t
@@ -2086,7 +2097,7 @@ On failure, return negative error code."
       nil)))
 	
 (defun wnn7-server-predict-save-data ()
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-save-datalist env)))
     (if (= result 0)
 	t
@@ -2094,7 +2105,7 @@ On failure, return negative error code."
       nil)))
 
 (defun wnn7-server-predict-set-timeinfo (yosokuselect throughyosoku inputtime keylen)
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-set-timeinfo env yosokuselect 
 					      throughyosoku inputtime keylen)))
     (if (= result 0)
@@ -2103,7 +2114,7 @@ On failure, return negative error code."
       nil)))
 
 (defun wnn7-server-predict-set-user-inputinfo (allkey userkey yosokuselect)
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-set-user-inputinfo env allkey userkey
 						    yosokuselect)))
     (if (= result 0)
@@ -2112,7 +2123,7 @@ On failure, return negative error code."
       nil)))
 
 (defun wnn7-server-predict-status ()
-  (let* ((env (if env-normal wnn7-env-norm wnn7-env-rev))
+  (let* ((env wnn7-env-norm)
 	 (result (wnn7rpc-yosoku-status env)))
     result))
   
