@@ -56,12 +56,106 @@
 
 ;;; Commentary:
 
+;;; 2002/5/16  XEmacs$B$K$*$$$F%f!<%6<-=q$N%/%i%$%"%s%HE>Aw$,$G$-$J$$LdBj$NBP1~(B
 ;;; 2001/9/30  Modified Error Messages
 
 ;;; Code:
 
+(defun wnn-perform-replace (from-string replacements)
+  "perform-replace $B$h$jITI,MW$J%*%W%7%g%s!"=hM}!"%(%3!<NN0h$X$NI=<($r:o=|$7$?$b$N(B"
+  (let* ((event (make-event))
+	 (nocasify (not (and case-fold-search case-replace
+			     (string-equal from-string
+					   (downcase from-string)))))
+	 (literal t)
+	 (search-function (if (boundp 'replace-search-function)
+			      replace-search-function
+			    'search-forward))
+	 (search-string from-string)
+	 (real-match-data nil)		; the match data for the current match
+	 (next-replacement nil)
+	 (replacement-index 0)
+	 (keep-going t)
+	 (stack nil)
+	 (next-rotate-count 0)
+	 (replace-count 0)
+	 (repeat-count 1)
+	 (lastrepl nil)			;Position after last match considered.
+	 ;; If non-nil, it is marker saying where in the buffer to
+	 ;; stop.
+	 (limit nil)
+	 (match-again t)
+	 ;; XEmacs addition
+	 (qr-case-fold-search
+	  (if (and case-fold-search search-caps-disable-folding)
+	      (if (boundp 'no-upper-case-p)
+		  (no-upper-case-p search-string nil)
+		(isearch-no-upper-case-p search-string))
+	    case-fold-search))
+	 )
+    ;; If the region is active, operate on region.
+    (when (region-active-p)
+      ;; Original Per Abrahamsen's code simply narrowed the region,
+      ;; thus providing a visual indication of the search boundary.
+      ;; Stallman, on the other hand, handles it like this.
+      (setq limit (copy-marker (region-end)))
+      (goto-char (region-beginning))
+      (zmacs-deactivate-region))
+    (if (stringp replacements)
+	(setq next-replacement replacements)
+      (or repeat-count (setq repeat-count 1)))
+    (push-mark)
+    (undo-boundary)
+    (unwind-protect
+	;; Loop finding occurrences that perhaps should be replaced.
+	(while (and keep-going
+		    (not (eobp))
+		    (or (null limit) (< (point) limit))
+		    (let ((case-fold-search qr-case-fold-search))
+		      (if (> emacs-minor-version 1)
+			  (funcall search-function search-string limit)
+			(funcall search-function search-string limit t)))
+		    ;; If the search string matches immediately after
+		    ;; the previous match, but it did not match there
+		    ;; before the replacement was done, ignore the match.
+		    (if (or (eq lastrepl (point)))
+			(if (or (eobp)
+				(and limit (>= (point) limit)))
+			    nil
+			  ;; Don't replace the null string
+			  ;; right after end of previous replacement.
+			  (forward-char 1)
+			  (let ((case-fold-search qr-case-fold-search))
+			    (if (> emacs-minor-version 1)
+				(funcall search-function search-string limit)
+			      (funcall search-function search-string limit t))))
+		      t))
 
-(require 'wnn7egg-comx21)
+	  ;; Save the data associated with the real match.
+	  (setq real-match-data (match-data))
+
+	  ;; If time for a change, advance to next replacement string.
+	  (if (and (listp replacements)
+		   (= next-rotate-count replace-count))
+	      (progn
+		(setq next-rotate-count
+		      (+ next-rotate-count repeat-count))
+		(setq next-replacement (nth replacement-index replacements))
+		(setq replacement-index (% (1+ replacement-index) (length replacements)))))
+	  (store-match-data real-match-data)
+	  (replace-match next-replacement nocasify literal)
+	  (setq replace-count (1+ replace-count))
+	  (setq lastrepl (point)))
+      ;; Useless in XEmacs.  We handle (de)highlighting through
+      ;; perform-replace-next-event.
+      ;(replace-dehighlight)
+      )
+    (or unread-command-events)
+    (and keep-going stack)))
+
+(if (> emacs-minor-version 1)
+    (require 'wnn7egg-comx214)
+  (require 'wnn7egg-comx21))
 
 (defmacro wnn-file-string ()
   (string-as-unibyte (decode-coding-string "$B#W#n#n$N%U%!%$%k(B" 'euc-jp)))
@@ -439,6 +533,67 @@
     (or (and (< errno (length msg)) (aref msg errno))
 	(format "#%d" errno))))
 
+;---
+(defun insert-file-contents-wnn (filename &optional visit start end replace)
+  (let (return-val coding-system used-codesys)
+    ;; OK, first load the file.
+    (condition-case err
+	(progn
+	  (run-hook-with-args 'insert-file-contents-access-hook
+			      filename visit)
+	  ;; determine the coding system to use, as described above.
+	  (setq coding-system 'binary)
+	  (if (consp coding-system)
+	      (setq return-val coding-system)
+	    (if (null (find-coding-system coding-system))
+		(progn
+		  (message
+		   "Invalid coding-system (%s), using 'undecided"
+		   coding-system)
+		  (setq coding-system 'undecided)))
+	    (setq return-val
+		  (insert-file-contents-internal filename visit start end
+						 replace coding-system
+						 ;; store here!
+						 'used-codesys))
+	    ))
+      (file-error
+       (run-hook-with-args 'insert-file-contents-error-hook
+			   filename visit err)
+       (signal (car err) (cdr err))))
+    (setq coding-system used-codesys)
+    ;; call any `post-read-conversion' for the coding system that
+    ;; was used ...
+    (let ((func
+	   (coding-system-property coding-system 'post-read-conversion))
+	  (endmark (make-marker)))
+      (set-marker endmark (+ (point) (nth 1 return-val)))
+      (if func
+	  (unwind-protect
+	      (save-excursion
+		(let (buffer-read-only)
+		  (funcall func (point) (marker-position endmark))))
+	    (if visit
+		(progn
+		  (set-buffer-auto-saved)
+		  (set-buffer-modified-p nil)))))
+      (setcar (cdr return-val) (- (marker-position endmark) (point))))
+    ;; now finally set the buffer's `buffer-file-coding-system'.
+    (if (run-hook-with-args-until-success 'insert-file-contents-post-hook
+					  filename visit return-val)
+	nil
+      (if (local-variable-p 'buffer-file-coding-system (current-buffer))
+	  ;; if buffer-file-coding-system is already local, just
+	  ;; set its eol type to what was found, if it wasn't
+	  ;; set already.
+	  (set-buffer-file-coding-system
+	   (subsidiary-coding-system buffer-file-coding-system
+				     (coding-system-eol-type coding-system)))
+	;; otherwise actually set buffer-file-coding-system.
+	(set-buffer-file-coding-system coding-system)))
+    return-val))
+;---
+
 (defmacro wnn7rpc-call-with-proc (proc vlist send-expr &rest receive-exprs)
   `(comm-call-with-proc ,proc
        ((zhuyin nil)
@@ -1231,18 +1386,74 @@ HINSHI and FUZOKUGO are information of preceding bunsetsu."
 		 env-id dic hinshi-table)
     (wnn7rpc-get-result)))
 
+(defmacro with-current-buffer-bin (buffer &rest body)
+  "Temporarily make BUFFER the current buffer and execute the forms in BODY.
+The value returned is the value of the last form in BODY.
+See also `with-temp-buffer'."
+  `(save-current-buffer
+    (set-buffer ,buffer)
+    (let ((coding-system-for-read 'binary)
+	  (coding-system-for-write 'binary))
+      (set-buffer-multibyte nil)
+      ,@body)))
+
+(defmacro with-temp-buffer-bin (&rest forms)
+  "Create a temporary buffer, and evaluate FORMS there like `progn'.
+See also `with-temp-file' and `with-output-to-string'."
+  (let ((temp-buffer (make-symbol "temp-buffer")))
+    `(let ((,temp-buffer
+            (get-buffer-create (generate-new-buffer-name " *temp*"))))
+       (unwind-protect
+	   (let ((coding-system-for-read 'binary)
+		 (coding-system-for-write 'binary))
+	     (set-buffer-multibyte nil)
+	     (with-current-buffer-bin ,temp-buffer
+	       ,@forms))
+         (and (buffer-name ,temp-buffer)
+              (kill-buffer ,temp-buffer))))))
+
 (defmacro wnn7rpc-with-temp-buffer (&rest body)
-  `(with-temp-buffer
-     (let ((coding-system-for-read 'no-conversion)
-	   (coding-system-for-write 'no-conversion))
+;  `(with-temp-buffer
+;     (let ((coding-system-for-read 'binary)
+;	   (coding-system-for-write 'binary))
+;       (set-buffer-multibyte nil)
+;       ,@body)))
+  `(with-temp-buffer-bin
+     (let ((coding-system-for-read 'binary)
+	   (coding-system-for-write 'binary))
        (set-buffer-multibyte nil)
        ,@body)))
 
+(defmacro with-temp-file-bin (file &rest forms)
+  (let ((temp-file (make-symbol "temp-file"))
+        (temp-buffer (make-symbol "temp-buffer")))
+    `(let ((,temp-file ,file)
+           (,temp-buffer
+            (get-buffer-create (generate-new-buffer-name " *temp file*"))))
+       (unwind-protect
+           (prog1
+               (with-current-buffer ,temp-buffer
+                 ,@forms)
+             (with-current-buffer ,temp-buffer
+               (widen)
+	       (let ((coding-system-for-read 'binary)
+		     (coding-system-for-write 'binary))
+		 (write-region (point-min) (point-max) ,temp-file nil 0))))
+         (and (buffer-name ,temp-buffer)
+              (kill-buffer ,temp-buffer))))))
+
 (defmacro wnn7rpc-with-write-file (filename error-handler &rest body)
+;  `(condition-case error
+;       (with-temp-file ,filename
+;	 (let ((coding-system-for-read 'binary)
+;	       (coding-system-for-write 'binary))
+;	   (set-buffer-multibyte nil)
+;	   ,@body))
+;     (file-error ,error-handler)))
   `(condition-case error
-       (with-temp-file ,filename
-	 (let ((coding-system-for-read 'no-conversion)
-	       (coding-system-for-write 'no-conversion))
+       (with-temp-file-bin ,filename
+	 (let ((coding-system-for-read 'binary)
+	       (coding-system-for-write 'binary))
 	   (set-buffer-multibyte nil)
 	   ,@body))
      (file-error ,error-handler)))
@@ -1286,7 +1497,7 @@ HINSHI and FUZOKUGO are information of preceding bunsetsu."
   (wnn7rpc-with-temp-buffer
     (if (null (file-readable-p filename))
 	(list nil (make-string (wnn-const WNN_UNIQ_LEN) 0) "" "")
-      (insert-file-contents filename nil 0 (wnn-const WNN_FILE_HEADER_LEN))
+      (insert-file-contents-internal filename nil 0 (wnn-const WNN_FILE_HEADER_LEN))
       (wnn7rpc-scan-file-header))))
 
 (defun wnn7rpc-check-local-file (path &optional preserve)
@@ -1333,7 +1544,7 @@ HINSHI and FUZOKUGO are information of preceding bunsetsu."
 (defun wnn7rpc-change-file-uniq (header path &optional new)
   (wnn7rpc-with-write-file path
       nil
-    (insert-file-contents path)
+    (insert-file-contents-internal path)
     (if (wnn7rpc-scan-file-header)
 	(let ((uniq (wnn7rpc-make-uniq (file-attributes path))))
 	  (goto-char (1+ (wnn-const WNN_FILE_STRING_LEN)))
@@ -1427,8 +1638,11 @@ HINSHI and FUZOKUGO are information of preceding bunsetsu."
 		    (comm-format (u) (wnn-const WNN_ACK))
 		    (comm-unpack (B) contents))
 		  (insert contents)
+		  (save-excursion
+		    (goto-char (point-min))
+		    (wnn-perform-replace "\377\0" "\377"))
 		  (if (= result 2)
-		      (insert-file-contents local-filename nil (1- (point))))
+		      (insert-file-contents-internal local-filename nil (1- (point))))
 		  (save-excursion
 		    (set-buffer (process-buffer proc))
 		    (wnn7rpc-get-result)))))))))
@@ -1457,7 +1671,7 @@ HINSHI and FUZOKUGO are information of preceding bunsetsu."
 		  (comm-format (s B)
 			       (concat wnn-system-name "!" filename)
 			       (wnn7rpc-with-temp-buffer
-				 (insert-file-contents filename)
+				 (insert-file-contents-internal filename)
 				 (buffer-string)))
 		  (wnn7rpc-get-result
 		    (wnn7env-set-client-file env filename)
@@ -1543,7 +1757,7 @@ HINSHI and FUZOKUGO are information of preceding bunsetsu."
    ((null (file-readable-p filename)) (- (wnn-const WNN_FILE_READ_ERROR)))
    (t 
     (wnn7rpc-with-temp-buffer
-      (insert-file-contents filename nil 0 (1- (wnn-const WNN_PASSWD_LEN)))
+      (insert-file-contents-internal filename nil 0 (1- (wnn-const WNN_PASSWD_LEN)))
       (goto-char 1)
       (if (and (search-forward-regexp "[\0\n]" nil 0)
 	       (= (preceding-char) 0))
